package test.gov.grants.apply.applicant;

import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.ApplicantIntegrationSoapBindingStub;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.SubmitApplicationRequest;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.SubmitApplicationResponse;
import gov.grants.apply.util.Attachment; 

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.text.ParseException;
import java.util.Properties;

import javax.activation.DataHandler;

import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

import org.apache.axis.attachments.AttachmentPart;
import org.apache.axis.attachments.DynamicContentDataHandler;
import org.apache.axis.attachments.ManagedMemoryDataSource;
import org.apache.axis.client.Call;
import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;

/**
 * Junit Test Case for invoking the Grants.gov SubmitApplicant Web Service that
 * utilizes DIME streaming for attachment encapsulation. DIME streaming allows
 * attachments of unkown sizes to be streamed to the server. This allows
 * attachments to be buffered into memory using DIME packets thus improving
 * performance and scalability.
 * 
 * @author Brian Husted
 * @July 08, 2005
 */
public class WSTestClient extends TestCase {

	private static Properties props = null;

	public static final String PROP_FILE = "/resources/application.properties";

	static final Logger log = Logger.getLogger(WSTestClient.class.getName());

	private static ApplicantIntegrationSoapBindingStub stub = null;

	public static final String LOG_4J_CONFIG_FILE = "log4j.xml";

	public static final String RESOURCES_LOG_4J_CONFIG_FILE = "/resources/"
			+ LOG_4J_CONFIG_FILE;

	
	public static final String GRANT_XML = "/resource/GrantApplication.xml";
	public static final String LARGE_ATTACHMENT = "/resources/LargeAttachment.zip";
	
	public static final String LARGE_ATTACHMENT_CONTENT_ID = "<LargeAttachment.zip>";

	//Properties from application.properties in /resources /////////////

	public static final String SOAP_HOST = "SOAP_HOST";

	public static final String SOAP_PORT = "SOAP_PORT";
	
	public static final String MAX_MEMORY_TO_BUFFER = "MAX_MEMORY_TO_BUFFER";

	////////////////////////////////////////////////////////////////////
	
	public static final String CONTENT_TYPE = "application/octet-stream";

	static final int BUFFER = 4096;

	static {
		init();
	}

	/**
	 * Constructor
	 * 
	 * @param str
	 */
	public WSTestClient(String str) {
		super(str);
	}

	private static void init() {
		try {
			DOMConfigurator.configure(WSTestClient.class
					.getResource(RESOURCES_LOG_4J_CONFIG_FILE));
			log
					.debug("log4J Config XML file loaded from /resources on classpath");

			System.setProperty("javax.net.ssl.keyStore",
					getProperty("javax.net.ssl.keyStore"));
			System.setProperty("javax.net.ssl.keyStorePassword",
					getProperty("javax.net.ssl.keyStorePassword"));
			System.setProperty("javax.net.ssl.trustStore",
					getProperty("javax.net.ssl.trustStore"));
			System.setProperty("javax.net.ssl.trustStorePassword",
					getProperty("javax.net.ssl.trustStorePassword"));

			String wsURL = getProperty(SOAP_HOST) + getProperty(SOAP_PORT);
			if (wsURL == null) {
				throw new RuntimeException(
						"SOAP HOST URL is not defined in props");
			}
			java.net.URL endpoint = new URL(wsURL);
			stub = new ApplicantIntegrationSoapBindingStub(endpoint, null);

		} catch (Exception ex) {
			log.fatal("Unable to initialize", ex);
			throw new RuntimeException(ex);
		}

	}

	/**
	 * Invokes the Grants.gov SubmitApplicationList Web Service using DIME
	 * streaming for attachments.
	 * 
	 * @throws Exception
	 */
	public static void testSubmitApplication() throws Exception {
		SubmitApplicationRequest submitRequest = new SubmitApplicationRequest();

		//submitRequest.setGrantApplicationXML(xml);

		SubmitApplicationResponse submitResponse = stub
				.submitApplication(submitRequest);
	}

	private void attach(ApplicantIntegrationSoapBindingStub stub,
			InputStream attachmentStream) throws Exception {

		AttachmentPart attachmentPart = new AttachmentPart();
		
		int max_memory = 10; 
		try{ 
			max_memory = Integer.parseInt( getProperty( MAX_MEMORY_TO_BUFFER ) );
		}catch( NumberFormatException ne ){}
		
		max_memory = (max_memory * 1024 * 1024);
		
		
		
		ManagedMemoryDataSource mgMemoryDataSource 
		 	= new ManagedMemoryDataSource(attachmentStream, max_memory, CONTENT_TYPE);  
		DynamicContentDataHandler dynamicDataHandler = 
			new DynamicContentDataHandler( mgMemoryDataSource );

		/**
		 * Tell the stub that the message being formed also contains an
		 * attachment, and it is defaulted to DIME encoding.
		 */

		stub._setProperty(Call.ATTACHMENT_ENCAPSULATION_FORMAT,
				Call.ATTACHMENT_ENCAPSULATION_FORMAT_DIME);
		attachmentPart.setDataHandler(dynamicDataHandler);

		attachmentPart.setContentId(LARGE_ATTACHMENT_CONTENT_ID);
		attachmentPart.setContentType(CONTENT_TYPE);

		stub.addAttachment(attachmentPart);

	}

	/**
	 * Get a property value from the soap server properties file.
	 * 
	 * @param key
	 *            An key value
	 * @return The property value or null if no property exists
	 * @throws IOException
	 */
	private static String getProperty(String key) throws IOException {
		if (props == null) {
			synchronized (WSTestClient.class) {
				if (props == null) {
					props = loadProperties();
				}
			}

		}

		return props.getProperty(key).trim();

	}

	/**
	 * Load Properties
	 * 
	 * @return The Properties
	 * @throws IOException
	 */
	private static Properties loadProperties() throws IOException {
		Properties props = new Properties();

		InputStream stream = null;
		try {
			stream = WSTestClient.class.getResourceAsStream(PROP_FILE);
			log.debug("application.properties file loaded from " + PROP_FILE
					+ " on classpath");

			props.load(stream);

		} finally {
			try {
				stream.close();
			} catch (Exception ex) {
			}
		}
		return props;
	}

	public static void main(String[] args) throws Exception {
		TestSuite suite = new TestSuite();
		suite.addTest(new WSTestClient("Grants.gov WS Test Client"));
		suite.run(new TestResult());

	}

}